import { Component, ViewChild } from '@angular/core';
import { EventModel, ResourceTimeRangeModel } from '@bryntum/schedulerpro';
import { BryntumSchedulerProComponent } from '@bryntum/schedulerpro-angular';
import { delay, of, shareReplay, Subject, switchMap } from 'rxjs';

@Component({
  selector: 'app-root',
  templateUrl: './app.component.html',
  styleUrls: ['./app.component.css'],
})
export class AppComponent {
  public config = {
    features: {
      tree: true,
      stripe: true,
      sort: false,
      taskEdit: false,
      scheduleTooltip: false,
      scheduleMenu: false,
      cellMenu: false,
      eventDragCreate: false,
      eventTooltip: {
        disabled: true,
      },
      eventMenu: true,
      eventResize: false,
      timeRanges: {
        showCurrentTimeLine: true,
      },
      resourceTimeRanges: {
        enableMouseEvents: true,
      },
      eventDrag: {
        showTooltip: false,
        constrainDragToTimeSlot: true,
        constrainDragToTimeline: false
      },
    },
    columns: [
      {
        width: 200,
        type: 'tree',
        field: 'name',
        editor: false,
      },
    ],
    viewPreset: {
      timeResolution: {
        unit: 'minute',
        increment: 15,
      },
      headers: [
        {
          unit: 'day',
          dateFormat: 'DD MMMM',
        },
        {
          unit: 'minute',
          increment: 60,
          dateFormat: 'HH:mm',
        },
      ],
    },
  };

  public timeZone = 'Asia/Aqtau';
  public startDate = '2023-01-05T00:00:00.000Z';
  public endDate = '2023-01-05T23:59:59.999Z';
  public resourceTimeRanges: ResourceTimeRangeModel[] = [];

  public loadDataSubject = new Subject<void>();
  public data$ = this.loadDataSubject.pipe(
    switchMap(() => {
      return of(this._generateData()).pipe(delay(5000));
    }),
    shareReplay(1)
  );

  private _generateData() {
    const resources = [];
    const events = [];
    for (let i = 0; i < 100; i++) {
      const resourceId = `North:${i}`;

      resources.push({
        id: resourceId,
        name: `Testing name - ${i}`,
      });

      events.push({
        id: i,
        manuallyScheduled: true,
        resourceId,
        startDate: '2023-01-05T18:00:00.000Z',
        endDate: '2023-01-05T21:00:00.000Z',
      });
    }
    return {
      events,
      resources,
    };
  }

  public loadDataFromServer() {
    this.loadDataSubject.next();
  }

  onEventSelectionChange(event: any, resources: any[]) {
    this.resourceTimeRanges = [];
    const selectedEvent: EventModel = event.selected[0];
    if (selectedEvent) {
      resources.forEach((rss) =>
        this.resourceTimeRanges.push(
          this.createResourceTimerange(selectedEvent, rss)
        )
      );
    }
  }

  createResourceTimerange(
    timeRange: { startDate: Date | string; endDate: Date | string },
    rss: any
  ) {
    return new ResourceTimeRangeModel({
      resourceId: rss.id,
      timeRangeColor: 'violet',
      style: 'height: 100%',
      startDate: timeRange.startDate,
      endDate: timeRange.endDate,
    });
  }
}
